/*
 * Copyright (c) 2002, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef __PATHANTYPEINFO_HPP
#define __PATHANTYPEINFO_HPP

#include <pathan/Pathan.hpp>
#include <xercesc/dom/DOMTypeInfo.hpp>

class XPath2MemoryManager;

/**
 * The <code>PathanTypeInfo</code> interface extends the <code>DOMTypeInfo</code>
 * @since DOM Level 3
 */
class PATHAN_EXPORT PathanTypeInfo: public XERCES_CPP_NAMESPACE_QUALIFIER DOMTypeInfo
{
public:
  // -----------------------------------------------------------------------
  //  Constructors
  // -----------------------------------------------------------------------
  /** @name Constructors */
  //@{
  /**
   * Constructor which takes a namespace URI and a name.
   *
   * @param uri The type namespace uri
   * @param name The type name 
   * @param memMgr The memory manager to use
   */
  PathanTypeInfo(const XMLCh* uri, const XMLCh* name, XPath2MemoryManager* memMgr);

  //@}

  // -----------------------------------------------------------------------
  //  Destructors
  // -----------------------------------------------------------------------
  /** @name Destructor */
  //@{
  /**
   * Destructor for PathanTypeInfo.
   *
   */
  virtual ~PathanTypeInfo();
  //@}
  
    //@{
    // -----------------------------------------------------------------------
    //  Getter methods
    // -----------------------------------------------------------------------
    /**
     * Returns The name of a type 
     *
     * @return The name of a type 
     * @since DOM level 3
     */
    virtual const XMLCh* getName() const;

    /**
     * The namespace of the type or null if no namespace information is available.
     *
     * @return The namespace of the type or null if no namespace information is available.
     * @since DOM level 3
     */
    virtual const XMLCh* getNamespace() const;
    //@}

  
private:
  const XMLCh* _uri;
  const XMLCh* _name;
};//PathanTypeInfo

#endif //__PATHANTYPEINFO_HPP

