/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
  Factory base class
*/

#ifndef _DATATYPEFACTORY_HPP
#define _DATATYPEFACTORY_HPP

#include <pathan/Pathan.hpp>
#include <pathan/internal/m_apm.h>
#include <pathan/AnyAtomicType.hpp>

class DynamicContext;
class XPath2MemoryManager;
class AnyAtomicType;
class ATAnySimpleType;
class ATAnyURIOrDerived;
class ATBase64BinaryOrDerived;
class ATBooleanOrDerived;
class ATDateOrDerived;
class ATDateTimeOrDerived;
class ATDecimalOrDerived;
class ATDoubleOrDerived;
class ATDurationOrDerived;
class ATFloatOrDerived;
class ATGDayOrDerived;
class ATGMonthOrDerived;
class ATGMonthDayOrDerived;
class ATGYearMonthOrDerived;
class ATGYearOrDerived;
class ATHexBinaryOrDerived;
class ATNotationOrDerived;
class ATQNameOrDerived;
class ATStringOrDerived;
class ATTimeOrDerived;
class ATUntypedAtomic;

class PATHAN_EXPORT DatatypeFactory
{
public:
  DatatypeFactory();
  virtual ~DatatypeFactory();

  ///Create instance of a datatype, using DatatypeFactoryTemplate to determine type
  virtual AnyAtomicType* createInstance(XPath2MemoryManager *xm,
                                        const XMLCh* typeURI,
                                        const XMLCh* typeName,
                                        const XMLCh* value,
                                        const DynamicContext* context) const = 0; 

  virtual bool checkInstance(const XMLCh* typeURI,
                             const XMLCh* typeName,
                             const XMLCh* value,
                             const DynamicContext* context) const = 0;

  virtual AnyAtomicType::AtomicObjectType getPrimitiveTypeIndex() const = 0;

  virtual const XMLCh* getPrimitiveTypeName() const = 0;
  //virtual const XMLCh* getPrimitiveTypeURI() const;

	class PATHAN_EXPORT POD2AT
	{
	public:
		static const ATDoubleOrDerived* createDouble(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context);
		static const ATFloatOrDerived* createFloat(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context);
		static const ATDecimalOrDerived* createInteger(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context);
		static const ATDecimalOrDerived* createDecimal(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context);
		static const ATDecimalOrDerived* createNonNegativeInteger(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context);
		static const ATDecimalOrDerived* createDecimalOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const MAPM value, const DynamicContext* context);
		static const ATDoubleOrDerived* createDoubleOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const MAPM value, const DynamicContext* context);
		static const ATFloatOrDerived* createFloatOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const MAPM value, const DynamicContext* context);
		static const ATBooleanOrDerived* createBoolean(XPath2MemoryManager *xm, bool value, const DynamicContext* context);

		static const ATBooleanOrDerived* createBooleanOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, bool value, const DynamicContext* context);
	};

	class PATHAN_EXPORT STR2AT
	{
	public:
		static const ATDoubleOrDerived* createDouble(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATFloatOrDerived* createFloat(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATDecimalOrDerived* createInteger(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATDecimalOrDerived* createDecimal(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATBooleanOrDerived* createBoolean(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATDateOrDerived* createDate(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATDateTimeOrDerived* createDateTime(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATTimeOrDerived* createTime(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATDurationOrDerived* createDayTimeDuration(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATDurationOrDerived* createYearMonthDuration(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATAnyURIOrDerived* createAnyURI(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATQNameOrDerived* createQName(XPath2MemoryManager *xm, const XMLCh* uri, const XMLCh* name, const DynamicContext* context);
		static const ATStringOrDerived* createString(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATBase64BinaryOrDerived* createBase64Binary(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATHexBinaryOrDerived* createHexBinary(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const AnyAtomicType* createDerivedFromAtomicType(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);

		static const ATAnySimpleType* createAnySimpleType(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
		static const ATAnyURIOrDerived* createAnyURIOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context); 
		static const ATBase64BinaryOrDerived* createBase64BinaryOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATBooleanOrDerived* createBooleanOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATDateOrDerived*  createDateOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATDateTimeOrDerived* createDateTimeOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATDecimalOrDerived* createDecimalOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATDoubleOrDerived* createDoubleOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATDurationOrDerived* createDurationOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATFloatOrDerived* createFloatOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATGDayOrDerived* createGDayOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);  
		static const ATGMonthDayOrDerived* createGMonthDayOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATGMonthOrDerived* createGMonthOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATGYearMonthOrDerived* createGYearMonthOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATGYearOrDerived* createGYearOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATHexBinaryOrDerived* createHexBinaryOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATNotationOrDerived* createDerivedFromNotation(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATQNameOrDerived* createQNameOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATQNameOrDerived* createQNameOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* uri, const XMLCh* name, const DynamicContext* context);
		static const ATStringOrDerived* createStringOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATTimeOrDerived* createTimeOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
		static const ATUntypedAtomic* createUntypedAtomic(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context);
	};

};

#endif  // DATATYPEFACTORY

